<?php namespace MTTPlugin\Lib\Classes;

/**
 * This class helps with extending WordPress user
 *
 * @since 3.0.5
 */
class User {

    /**
     * WordPress User instance WP_User
     *
     * @var WP_User
     *
     * @since    3.0.5
     */
    public $wp_user = null;

    /**
     * User meta
     *
     * @var    array
     *
     * @since    3.0.5
     */
    protected $user_meta = array();

    /**
     * User constructor.
     *
     * @param null $user
     *
     * @since    3.0.5
     */
    public function __construct($user = null) {
        if($user === null) {
            $user = \wp_get_current_user();
        } if(is_a($user, __NAMESPACE__ . '\User')) {
            $user = $user->getWPUser();
        } elseif(!is_a($user, 'WP_User')) {
            $user = \get_user_by('id', (int) $user);
        }
        if(is_a($user, 'WP_User')) {
            $this->wp_user = $user;
        } else {
            $this->wp_user = new \WP_User();
        }
    }

    /**
     * Does the user exists
     *
     * @return bool
     *
     * @since    3.0.5
     */
    public function exist() {
        if(is_a($this->getWPUser(), 'WP_User') && $this->ID > 0) {
            return true;
        }
        return false;
    }

    /**
     * Return WP_User object
     *
     * @return WP_User|null
     *
     * @since    1.0.0
     */
    public function getWPUser() {
        return $this->wp_user;
    }

    /**
     * Get Role (primary/first)
     *
     * @return string
     *
     * @since    3.0.5
     */
    public function getRole() {
        return isset($this->roles[0]) ? $this->roles[0] : '';
    }

    /**
     * Get Roles
     *
     * @return mixed
     *
     * @since    3.0.5
     */
    public function getRoles() {
        return $this->roles;
    }

    /**
     * Get User ID
     *
     * @return mixed
     *
     * @since    3.0.5
     */
    public function getID() {
        return $this->ID;
    }

    /**
     * Get user meta
     *
     * @param      $name
     * @param null $default
     * @param bool $single
     *
     * @return null
     *
     * @since    3.0.5
     */
    public function getMeta($name, $default = null, $single = true) {
        $meta = \get_user_meta($this->ID, $name, $single);

        if(!empty($meta)) {
            return $meta;
        }

        return $default;
    }

    public function setMeta($name, $value) {
        return \update_user_meta($this->getID(), $name, $value);
    }

    // MAGIC METHODS
    public function __call($name, $arguments) {
        return call_user_func_array(array($this->getWPUser(), $name), $arguments);
    }

    public function __get($name) {
        return $this->getWPUser()->{$name};
    }
}