<?php namespace MTTPlugin\Lib\Tools\Admin\Notice;

use MTTPlugin\Lib\Tools\Loader;

class NoticeFactory {

	use \MTTPlugin\Lib\Traits\Templating;

	protected static $initialized = false;
	protected static $notices = array();
	protected static $notices_to_save = array();

	public static function init() {
		if(!static::$initialized) {
			static::registerHooks();
			static::loadNotices();
			static::$initialized = true;
		}
	}

	private static function registerHooks() {
		Loader::addAction('admin_notices', __CLASS__, 'displayNotices');
		Loader::addAction('admin_enqueue_scripts', __CLASS__, 'registerScript');
		Loader::addAction('wp_ajax_mtt_admin_notice_dismiss' , __CLASS__, 'dismissNotice');
	}

	private static function loadNotices() {
		static::$notices = \get_option('mtt/admin/notices', array());
	}

	public static function dismissNotice() {
		static::removeNotice($_GET['notice']);
		die();
	}

	public static function removeNotice($id) {
		if(isset(static::$notices[$id])) {
			unset(static::$notices[$id]);
			static::saveNotices();
		}
	}

	public static function removeNoticeAfter($id, $displays = 1) {
		if($displays > 0) {
			if(isset(static::$notices[$id])) {
				static::$notices[$id]['remove_after'] = $displays;
				static::saveNotices();
			}
		} else {
			static::removeNotice($id);
		}
	}

	private static function saveNotices() {
		\update_option('mtt/admin/notices', static::$notices);
	}

	public static function displayNotices() {
		foreach(static::$notices as $notice) {
			static::getTemplate('notice', array('notice' => $notice));
			if(isset($notice['remove_after']) && $notice['remove_after'] !== false) {
				static::removeNoticeAfter($notice['id'], --$notice['remove_after']);
			}
		}
	}

	public static function registerScript() {
		if(!empty(static::$notices)) {
			static::addScript('notices.js');
		}
	}

	public static function addNotice($class, $message, $remove_after = 1, $is_dismissible = false, $id = '') {
		$notice_id = $id !== '' ? $id : uniqid();

		static::$notices[$notice_id] = array(
			'id' => $notice_id,
			'class' => $class,
			'message' => $message,
			'is_dismissible' => $is_dismissible,
			'remove_after' => $remove_after
		);

		if($remove_after > 0) {
			static::saveNotices();
		}
	}

	public static function addNoticeSuccess($message, $remove_after = 1, $is_dismissible = false, $id = '') {
		static::addNotice('notice-success', $message, $remove_after, $is_dismissible, $id);
	}

	public static function addNoticeError($message, $remove_after = 1, $is_dismissible = false, $id = '') {
		static::addNotice('notice-error', $message, $remove_after, $is_dismissible, $id);
	}

	public static function addNoticeWarning($message, $remove_after = 1, $is_dismissible = false, $id = '') {
		static::addNotice('notice-warning', $message, $remove_after, $is_dismissible, $id);
	}

	public static function addNoticeInfo($message, $remove_after = 1, $is_dismissible = false, $id = '') {
		static::addNotice('notice-info', $message, $remove_after, $is_dismissible, $id);
	}
}