<?php namespace MTTPlugin\Lib\Tools\Admin\Settings;

use MTTPlugin\Lib\Tools\Loader;

trait SettingsFactory {

	protected static $settings_sections = array();

	protected static $settings_fields = array();

	protected static $plugin_admin_instance = null;

	private function settingsFactoryInit() {
		if(is_null(static::$plugin_admin_instance)) {
			static::$plugin_admin_instance = $this;
		}
		$this->settingsFactoryRegisterActions();
	}

	private function settingsFactoryRegisterActions() {
		Loader::addAction('admin_init', $this, 'registerSettings');
	}

	public function registerSettings() {
		foreach(static::$settings_sections as $section) {
			$section->registerSettings();
		}
		foreach(static::$settings_fields as $field) {
			$field->registerField();
		}
	}

	public static function addSettingsSection($id, $title, $page_slug, $args = array()) {
		$defaults = array(
			'id' => $id,
			'title' => $title,
			'page_slug' => $page_slug
		);

		$args = \wp_parse_args($args, $defaults);

		static::$settings_sections[$id] = new SettingsSection(static::$plugin_admin_instance, $args);

		return static::$settings_sections[$id];
	}

	public static function addSettingsField($id, $title, $page_slug, $section_id, array $args = array()) {
		$defaults = array(
			'id' => $id,
			'title' => $title,
			'page_slug' => $page_slug,
			'section_id' => $section_id,
			'type' => 'text',
		);

		$args = \wp_parse_args($args, $defaults);

		static::$settings_fields[$id] = new SettingsField(static::$plugin_admin_instance, $args);

		return static::$settings_fields[$id];
	}

	public static function addSettingsFields($page_slug, $section_id, array $settings_fields) {
		foreach($settings_fields as $field) {
			$defaults = array(
				'id' => 'settings-field',
				'title' => static::$plugin_admin_instance->translate('Settings Field'),
			);

			$field = \wp_parse_args($field, $defaults);

			static::addSettingsField($field['id'], $field['title'], $page_slug, $section_id, $field);
		}
	}

	public function registerSetting($settings_section, $option_name, $sanitize_component = null, $sanitize_callback = '') {
		\register_setting($settings_section, $option_name, (is_object($sanitize_component) ? array($sanitize_component, $sanitize_callback) : $sanitize_callback));
	}
}