<?php namespace MTTPlugin\Lib\Tools\Admin\Settings;

class SettingsField {

	private $id;
	private $title;
	private $component;
	private $callback;
	private $page_slug;
	private $section_id;
	private $type;
	private $args;
	private $default;

	private $plugin_admin;

	public function __construct($plugin_admin, $args) {
		$this->plugin_admin = $plugin_admin;

		$this->addSettingsFieldInfo($args);
		$this->loadFunctionsClass();
	}

	public function addSettingsFieldInfo(array $settings_field_info = array()) {
		$defaults = array(
			'id' => 'settings-field',
			'title' => $this->plugin_admin->translate('Settings Field'),
			'component' => $this,
			'callback' => 'displayField',
			'page_slug' => 'admin-page',
			'section_id' => 'settings-section',
			'content' => '',
			'type' => 'text',
			'args' => [],
			'default' => '',
		);

		$settings_field_info = \wp_parse_args($settings_field_info, $defaults);

		foreach($settings_field_info as $key => $info) {
			if(property_exists($this, $key)) {
				$this->{$key} = $info;
			}
		}
	}
	
	public function registerField() {
		\add_settings_field($this->id, $this->title, array($this->component, $this->callback), $this->page_slug, $this->section_id, $this->args);

		\register_setting($this->section_id, $this->id);
	}

	public function displayField() {
		$args = array(
			'id' => $this->id,
			'old_value' => \get_option($this->id, $this->default),
		);

		$args = \apply_filters('mtt_settings_field_args', $args, $this);

		$this->plugin_admin->getTemplate('settings-fields.' . $this->type, $args);
	}

	public function loadFunctionsClass() {
		$class = str_replace(' ', '', ucwords(str_replace(array('-', '_',), ' ', $this->type)));

		$class_path = '\SettingsFields\\' . $class;

		$plugin_class = $this->plugin_admin->getPluginInfo('namespace') . '\Plugin' . $class_path;
		$framework_class = __NAMESPACE__ . $class_path;

		if(class_exists($plugin_class)) {
			$class = new $plugin_class($this);
		} else if(class_exists($framework_class)) {
			$class = new $framework_class($this);
		}
	}

	public function getType() {
		return $this->type;
	}
}