<?php namespace MTTPlugin\Lib\Tools\Admin;

use MTTPlugin\Lib\Tools\Loader;

trait AdminPageFactory {

	protected static $admin_menu_pages = array();
	protected static $plugin_admin_instance = null;

	private function adminPageFactoryInit() {
		if(is_null(static::$plugin_admin_instance)) {
			static::$plugin_admin_instance = $this;
		}
		$this->adminPageFactoryRegisterActions();
	}

	private function adminPageFactoryRegisterActions() {
		Loader::addAction('admin_menu', $this, 'registerMenuPages');
	}

	public function registerMenuPages() {
		foreach(static::$admin_menu_pages as $page) {
			$page->registerMenuPage();
		}
	}

	public static function addMenuPage($title, $slug, $args = []) {
		$defaults = array(
			'page_title' => $title,
			'menu_title' => $title,
			'submenu_title' => $title,
			'menu_slug' => $slug,
			'type' => 'top-menu',
		);

		$args = \wp_parse_args($args, $defaults);

		static::$admin_menu_pages[$slug] = new AdminPage(static::$plugin_admin_instance, $args);

		return static::$admin_menu_pages[$slug];
	}

	public static function addSubmenuPage($title, $parent_slug, $slug, $args = []) {
		$defaults = array(
			'parent_slug' => $parent_slug,
			'page_title' => $title,
			'menu_title' => $title,
			'menu_slug' => $slug,
			'type' => 'sub-menu',
		);

		$args = \wp_parse_args($args, $defaults);

		static::$admin_menu_pages[$slug] = new AdminPage(static::$plugin_admin_instance, $args);

		return static::$admin_menu_pages[$slug];
	}

	public function addOptionsPage($title, $slug, $args = []) {
		$defaults = array(
			'parent_slug' => 'options-general.php',
		);

		$args = \wp_parse_args($args, $defaults);

		return $this->addSubmenuPage($title, $args['parent_slug'], $slug, $args);
	}
}