<?php namespace MTTPlugin\Lib\Tools;

class Autoloader {

	static $plugins = array();

	private $lib_namespace = 'MTTPlugin';
	private $plugin_namespace = 'MTTPlugin';
	private $plugin_path;
	private $version;

	public function __construct($plugin_path, $plugin_namespace, $version = '0.0.0') {

		$this->plugin_path = $plugin_path;
		$this->plugin_namespace = $plugin_namespace;
		$this->version = $version;

		static::$plugins[$this->plugin_path] = array(
			'version' => $this->version,
			'autoloader' => $this,
		);

		$this->registerActions();
	}

	private function registerActions() {
		if(!\has_action('plugins_loaded', array(__CLASS__, 'registerAll'))) {
			\add_action('plugins_loaded', array(__CLASS__, 'registerAll'), 1);
		}
		if(\did_action('plugins_loaded')) {
			static::registerAll();
		}
	}

	public function unregister() {
		spl_autoload_unregister(array($this, 'autoloader'));
	}

	public function register($prepend = true) {
		spl_autoload_register(array($this, 'autoloader'), true, $prepend);
	}

	public function autoloader($class) {
		$path_parts = explode('\\', $class);

		if(count($path_parts) <= 1) {
			return;
		}

		$start = array_shift($path_parts);

		if($start != $this->lib_namespace && $start != $this->plugin_namespace) {
			return;
		}

		$locations = array($this->plugin_path);

		$end = array_pop($path_parts);

		foreach($locations as $location) {
			foreach($path_parts as $part) {
				preg_match_all('!([A-Z][A-Z0-9]*(?=$|[A-Z][a-z0-9])|[A-Za-z][a-z0-9]+)!', $part, $matches);

				$parts = $matches[0];

				$part = implode('-', $parts);

				$location .= strtolower($part) . '/';
			}

			preg_match_all('!([A-Z][A-Z0-9]*(?=$|[A-Z][a-z0-9])|[A-Za-z][a-z0-9]+)!', $end, $matches);

			$file_name_start_strings = array(
				'class.',
				'abstract.',
				'trait.',
				'model.',
				''
			);

			if(count($matches[0]) > 1 && end($matches[0]) === 'Controller') {
				$file_name_start_strings = array('controller.');
				array_pop($matches[0]);
			}

			foreach($file_name_start_strings as $file_name_start_string) {
				$end = implode('-', $matches[0]);

				$file_ending = $file_name_start_string . strtolower($end) . '.php';

				if(is_file($location . $file_ending)) {
					include_once($location . $file_ending);

					return;
				}
			}

		}
	}

	/* STATIC FUNCTIONS */

	private static function sortPluginsArray($a, $b) {
		if($a['version'] == $b['version']) {
			return 0;
		}

		return $a['version'] < $b['version'] ? -1 : 1;
	}

	public static function registerAll() {
		usort(static::$plugins, 'static::sortPluginsArray');

		foreach(static::$plugins as $plugin_path => $info) {
			$info['autoloader']->register();
		}

		\do_action('mtt/framework/initialize/after');
	}
}