<?php

namespace RezipPackagingWoocommerce\Plugin\Controllers;

use MTTPlugin\Lib\Tools\Admin\AdminPageFactory;
use RezipPackagingWoocommerce\Plugin\Plugin;
use MTTPlugin\Lib\Abstracts\Controller;
use MTTPlugin\Lib\Tools\Loader;
use RezipPackagingWoocommerce\Plugin\PluginAdmin;

class PackagingController extends Controller{

	protected function registerActions() {
		parent::registerActions();

		Loader::addAction('woocommerce_after_cart_table', $this, 'displayPackagingChoice');
		Loader::addAction('woocommerce_checkout_before_order_review', $this, 'displayPackagingChoice');
		Loader::addAction('wp_ajax_nopriv_toggle_packaging_choice', $this, 'togglePackagingChoice');
		Loader::addAction('wp_ajax_toggle_packaging_choice', $this, 'togglePackagingChoice');
		Loader::addAction('woocommerce_cart_calculate_fees', $this, 'addFees');
		Loader::addAction('wp_enqueue_scripts', $this, 'enqueueScripts');
		Loader::addAction('wp_ajax_update_rezip_id', $this, 'updateRezipID');
	}

	protected function registerFilters() {
		parent::registerFilters();

		Loader::addFilter('is_protected_meta', $this, 'protectedMeta', 10, 2);
	}

	protected function registerActionsAdmin() {
		parent::registerActionsAdmin();

		Loader::addAction('admin_enqueue_scripts', $this, 'enqueueAdminScripts');

		Loader::addAction('manage_shop_order_posts_custom_column', $this, 'adminOrderColumnContent', 10, 2);


		Loader::addAction('add_meta_boxes', $this, 'addAdminOrderMetaBox');
		Loader::addAction('save_post', $this, 'saveOrderMetaBox', 10, 2);

		Loader::addAction('pre_get_posts', $this, 'rezipPackagingOrderBy');
	}

	protected function registerFiltersAdmin() {
		parent::registerFiltersAdmin();

		// Loader::addFilter('manage_edit-shop_order_columns', $this, 'addAdminOrderColumns');
		// Loader::addFilter('manage_edit-shop_order_sortable_columns', $this, 'addAdminOrderColumnsSortable');
	}

	public function rezipPackagingOrderBy($query){
		if(!is_admin()){
			return;
		}

		$orderby = $query->get('orderby');

		if('rezip_packaging_id' == $orderby){
			$query->set('meta_key', 're-zip_id');
			$query->set('orderby', 'meta_value_num');
		}
	}

	public function updateRezipID(){
		$post_id = sanitize_text_field($_POST['order_id']);
		$post = get_post($post_id);

		$this->saveOrderMetaBox($post_id, $post);
		$this->adminOrderColumnContent('rezip_packaging_id no-link', $post_id);
		exit();
	}

	public function protectedMeta($protected, $meta_key) {
		if($meta_key === 're-zip_id') {
			return true;
		}

		return $protected;
	}

	public function enqueueScripts()
	{
		if ( is_checkout() || is_cart() ) {
			$localizeArgs = array(
				'ajaxurl' => admin_url('admin-ajax.php'),
				'action' => 'toggle_packaging_choice'
			);

			Plugin::addStyle('packaging');
			Plugin::addScript('packaging');
			Plugin::localizeScript( 'packaging', 'ajax_object_rezip', $localizeArgs );

		}
	}

	public function enqueueAdminScripts() {
		$screen = get_current_screen();
		if ( in_array( $screen->id, [ 'shop_order', 'edit-shop_order' ] ) ) {
			PluginAdmin::addStyle('rezip-packaging');
			PluginAdmin::addScript('rezip-packaging');
			PluginAdmin::localizeScript('rezip-packaging', 'rezip', array('ajaxurl' => admin_url('admin-ajax.php')));
		}
	}

	public function displayPackagingChoice() {

		$args = [
			'fee_active' => $rezip = WC()->session->get('rezip')
		];

		if ( n02_is_display_rezip_offer() ) {
			Plugin::getTemplate( 'public.cart.packaging', $args );
		}

		// if ( get_option( 'rezip-type', 'fee' ) == 'fee' ) {
		// 	if ( n02_is_display_rezip_offer() ) {
		// 		Plugin::getTemplate( 'public.cart.packaging', $args );
		// 	}
		// } else {
		// 	if ( ! n02_is_package_in_cart() && n02_is_display_rezip_offer() ) {
		// 		Plugin::getTemplate( 'public.cart.packaging', $args );
		// 	}
		// }

	}

	public function togglePackagingChoice() {
		if ( isset( $_POST['addPackagingToOrder'] ) ) {
			if ( sanitize_text_field( $_POST['addPackagingToOrder'] ) == 'true' ) {

				echo 'Toggling Packaging on';
				WC()->session->set( 'rezip', true );
				if ( get_option( 'rezip-type' ) == 'product' ) {
					n02_add_to_cart();
				}

			} else {

				echo 'Toggling Packaging off';
				WC()->session->set( 'rezip', false );
				if ( get_option( 'rezip-type' ) == 'product' ) {
					n02_remove_from_cart();
				}

			}
		} else {
			echo 'Did not receive required parameters.';
		}
		wp_die();
	}

	public function addFees()
	{

		global $woocommerce;
		$rezip = WC()->session->get( 'rezip' );

		// wc_print_notice( date( 'H:i:s' ) . " - Rezip Session = " . $rezip, 'notice' );
		// wc_print_notice( $rezip );

		if ( isset( $rezip ) && $rezip === true ) {
			if ( get_option( 'rezip-type', 'fee' ) == 'fee' ) {
				$woocommerce->cart->add_fee(
					SettingsController::getFeeName(),
					SettingsController::getFeePriceExTax(),
					true,
					''
				);
			} else {
				// Move to
				// n02_add_to_cart();
			}
		}

	}

	public function addAdminOrderColumns($columns) {
		$columns['rezip_packaging_id no-link'] = Plugin::translate('Re-Zip Packaging ID');
		return $columns;
	}

	public function addAdminOrderColumnsSortable($columns) {
		$columns['rezip_packaging_id no-link'] = 'rezip_packaging_id';
		return $columns;
	}

	public function adminOrderColumnContent($column, $post_id) {
		if($column !== 'rezip_packaging_id no-link') {
			return;
		}

		$args = array(
			'rezip_id' => get_post_meta($post_id, 're-zip_id', true),
			'order_id' => $post_id,
			'order_has_packaging' => $this->orderHasPackaging($post_id)
		);

		Plugin::getTemplate('admin.order-column-rezip_packgaging_id', $args);
	}

	public function orderHasPackaging( $order )
	{

		if(!is_a($order, 'WC_Order')) {
			$order = new \WC_Order($order);
		}

		if(!empty(get_post_meta($order->get_id(), 'rezip_packaging', true))) {
			return true;
		}

		if(!empty(get_post_meta($order->get_id(), 're-zip_id', true))) {
			return true;
		}

		foreach($order->get_fees() as $fee) {
			if($fee->get_name() === SettingsController::getFeeName()) {
				return true;
			}
		}

		// as Product
		$items = $order->get_items();
		$rezip_product_id = get_option( 'rezip-settings-product-id' );
		foreach ( $items as $item ) {
			$product_id = $item->get_variation_id() ? $item->get_variation_id() : $item->get_product_id();
			if ( $product_id == $rezip_product_id ) {
				return true;
			}
		}

		return false;

	}

	public function addAdminOrderMetaBox() {
		add_meta_box('rezip_packgaging_id', Plugin::translate('Rezip Packaging ID'), array($this, 'adminOrderMetaBoxContent'), 'shop_order', 'side');
	}

	public function adminOrderMetaBoxContent($post) {
		$args = array(
			'rezip_id' => get_post_meta($post->ID, 're-zip_id', true),
			'order_id' => $post->ID,
			'order_has_packaging' => $this->orderHasPackaging($post->ID)
		);

		Plugin::getTemplate('admin.order-meta-box-rezip_packgaging_id', $args);
	}

	public function saveOrderMetaBox($post_id, $post)
    {
		if ( isset( $_POST['rezip_id_nonce'] ) ) {
			if (
				empty(sanitize_text_field($_POST['rezip_id_nonce'])) ||
				!wp_verify_nonce(sanitize_text_field( $_POST['rezip_id_nonce'] ), 'rezip_id_change') ||
				!current_user_can( 'edit_shop_order', $post_id) ||
				wp_is_post_autosave($post_id) ||
				wp_is_post_revision( $post_id )
			) {
				return;
			}
			update_post_meta($post_id, 're-zip_id', sanitize_text_field( $_POST['rezip_id'] ));
		}
	}

}