<?php

namespace RezipPackagingWoocommerce\Plugin\Controllers;

use MTTPlugin\Lib\Abstracts\Controller;
use MTTPlugin\Lib\Tools\Loader;
use RezipPackagingWoocommerce\Plugin\Plugin;
use RezipPackagingWoocommerce\Plugin\PluginAdmin;

class SettingsController extends Controller
{

	protected function registerFiltersAdmin()
	{
		parent::registerFiltersAdmin();
		Loader::addFilter('woocommerce_get_sections_advanced', $this, 'addSettingsPage');
		Loader::addFilter('woocommerce_get_settings_advanced', $this, 'addSettings', 10, 2);
		Loader::addFilter('woocommerce_admin_settings_sanitize_option', $this, 'saveEditorField', 10, 3);
	}

	protected function registerActionsAdmin()
	{
		parent::registerActions();

		Loader::addAction('woocommerce_admin_field_editor', $this, 'editorField');
		Loader::addAction('woocommerce_admin_field_image', $this, 'imageField');
	}

	public function enqueueScriptsAdmin()
	{
		wp_enqueue_media();
		PluginAdmin::addScript('arthur-image-uploader');
	}

	public function saveEditorField($value, $option, $raw_value)
	{
		if ($option['type'] === 'editor') {
			return $raw_value;
		}
		return $value;
	}

	public function editorField($value)
	{
		$option_value = \WC_Admin_Settings::get_option($value['id'], $value['default']);

		$field_description = \WC_Admin_Settings::get_field_description($value);

		PluginAdmin::getTemplate('settings-fields.editor', array('value' => $value, 'option_value' => $option_value, 'field_description' => $field_description,));
	}

	public function imageField($value)
	{
		wp_enqueue_media();
		PluginAdmin::addScript('settings-fields.image-uploader');
		$option_value = static::getSetting($value['id'], $value['default']);
		if (is_numeric($option_value)) {
			$_image_src = wp_get_attachment_image_src($option_value, array(150, 150, true));
			$option_value = $_image_src[0];
		}

		$field_description = \WC_Admin_Settings::get_field_description($value);

		PluginAdmin::getTemplate('settings-fields.image-uploader', array('value' => $value, 'option_value' => $option_value, 'field_description' => $field_description));
	}

	public function addSettingsPage($sections)
	{
		$sections['rezip_settings'] = Plugin::translate('Re-Zip Packaging Settings');

		return $sections;
	}

	public function product_link($post_id)
	{
		$post = get_post($post_id);
		$link = get_edit_post_link($post_id);
		return "
			Rezip Product Detected: <a href='{$link}' target='blank' >#{$post_id} {$post->post_title}</a>
			Update the product price using the field below.
		";
	}

	public function addSettings($settings, $current_section)
	{

		if ($current_section === 'rezip_settings') {

			$type = get_option('rezip-type', 'fee');

			$settings_rezip = [

				$this->addSettingField(
					'rezip-settings-page-title',
					'Re-Zip Packaging Settings',
					'title',
					'The following settings is used to configure the Re-Zip Packaging Plugin.'
				),

				[
					'name'    => 'Type',
					'id'      => 'rezip-type',
					'type'    => 'select',
					'options' => [
						'product' => 'Product',
						'fee'     => 'Fee',
					],
					'desc'    => 'Offer REZIP as a product or as an addon Fee.',
				],

				[
					'type' => 'sectionend',
					'id'   => 'rezip-settings-page-title',
				],

			];

			// Rezip - Product
			if (true) {

				$text = '<strong style="color:red;">No Re-Zip product detected.  Upon saving the settings a Re-ZIP Product will automatically be created</strong>';
				$product_id = get_option('rezip-settings-product-id');
				$product_wc = wc_get_product($product_id);

				if ($product_wc && $product_wc->get_status() != 'publish') {
					$product_wc = false;
				}

				// d( $product_id, $product_wc );

				if ($type == 'product') {

					// Create if not exist
					if (!$product_wc && $_POST['rezip-fee-price']) {
						if (!isset($_POST['flagRezipPass'])) {
							$product_id = n02_create_product_rezip();
							update_option('rezip-settings-product-id', $product_id);
							$_POST['flagRezipPass'] = true;
						}
					}

					// Set Price of REZIP Product
					if (isset($_POST['rezip-fee-price'])) {
						if ($_POST['rezip-type'] == 'product') {
							$product_id = get_option('rezip-settings-product-id');
							$product_wc = wc_get_product($product_id);
							$product_wc->set_regular_price($_POST['rezip-fee-price']);
							$product_wc->set_price($_POST['rezip-fee-price']);
							$product_wc->save();
						}
					}
				}

				if ($product_wc) {
					$text = $this->product_link($product_id);
				}

				$settings_product = [

					[
						'title' => 'Product Settings',
						'desc'  => $text,
						'type'  => 'title',
						'id'    => 'rezip-settings-product-title',
					],

					// [
					// 	'name'    => 'Product ID',
					// 	'id'      => 'rezip-settings-product-id',
					// 	'type'    => 'text',
					// ],

					['type' => 'sectionend', 'id' => 'rezip-settings-product-title'],

				];
			}

			// Cart & Checkout Settings
			if (true) {

				$settings_checkout = [

					[
						'title' => 'Cart and Checkout Settings',
						'desc'  => 'Customize the look and text of the Re-Zip offer which appear on the Cart and Checkout page.',
						'type'  => 'title',
						'id'    => 'rezip-settings-fee-title',
					],

					$this->addSettingField('rezip-image', 'Image', 'image', 'Image to show on the Cart and Checkout', Plugin::getImgURL('rezip_badge.png')),
					$this->addSettingField('rezip-title', 'Re-Zip title', 'text', 'Title of the Re-Zip box on Cart and Checkout', Plugin::translate('Nej tak til engangsemballage!')),
					$this->addSettingField('rezip-short-description', 'Short description', 'textarea', 'Short description of the Re-Zip box on Cart and Checkout', Plugin::translate('Vælg cirkulær emballage og få rabat ved næste køb.')),
					$this->addSettingField('rezip-description', 'Description', 'editor', 'Description showing when hover over "Read More"', Plugin::translate('<p>Sådan fungerer det:</p><ol><li>Tilvælg RE-ZIP´s cirkulære emballage.</li><li>Returner den tomme emballage gratis i en dao pakkeshop ved at scanne QR-koden på emballagen.</li><li>Modtag som tak en rabatkode.</li></ol>')),
					$this->addSettingField('rezip-checkbox-text', 'Checkbox text', 'text', 'Checkbox text on Cart and Checkout', Plugin::translate('Ja tak!')),
					$this->addSettingField('rezip-fee-name', 'Name', 'text', 'Packaging name shown on receipt, Cart and Checkout', Plugin::translate('Re-Zip emballage')),
					$this->addSettingField('rezip-fee-price', 'Price', 'number', 'Packaging price', 15),
					['type' => 'sectionend', 'id' => 'rezip-settings-fee-title'],

				];
			}

			// Restrictions
			if (true) {

				$settings_restrictions = [

					[
						'title' => 'Restrictions',
						'desc'  => __('Disable REZIP offers when the following criteria are met.'),
						'type'  => 'title',
						'id'    => 'rezip-restrictions-title',
						'class' => 'xxx',
					],

					[
						'title'           => 'Disable by User Role',
						'desc'            => 'Enable restriction by user role.',
						'id'              => 'rezip-disable-user-role',
						'default'         => 'no',
						'type'            => 'checkbox',
					],

					[
						'name'    => 'User Roles',
						'desc'    => 'Selected user roles / type will not trigger REZIP offers.',
						'id'      => 'rezip-disable-user-role-value',
						'type'    => 'multiselect',
						'options' => n02_get_user_roles(),
						'desc_tip' =>  true,
						// 	'css'     => 'min-width:150px;',
					],

					[
						'title'           => 'Disable by Max Weight',
						'desc'            => 'Enable restriction by reaching a certain weight in the cart.',
						'id'              => 'rezip-disable-weight',
						'default'         => 'no',
						'type'            => 'checkbox',
					],

					[
						'title'           => 'Weight',
						'desc'            => 'Disable REZIP offer if weight reaches the selected value',
						'id'              => 'rezip-disable-weight-value',
						'default'         => '10',
						'type'            => 'number',
						'desc_tip'        =>  true,
					],

					[
						'title'           => 'Disable by Product Tags',
						'desc'            => 'Enable restriction if product tag/s are included in the cart.',
						'id'              => 'rezip-disable-product-tags',
						'default'         => 'no',
						'type'            => 'checkbox',
					],

					[
						'name'    => 'Product Tags',
						'desc'    => 'Selected product tags will not trigger REZIP offers.',
						'id'      => 'rezip-disable-product-tags-value',
						'type'    => 'multiselect',
						'options' => n02_get_product_tags(),
						'desc_tip' =>  true,
					],

					['type' => 'sectionend', 'id' => 'rezip-restrictions-title'],


				];
			}

			$settings_rezip = array_merge($settings_rezip, $settings_product, $settings_checkout, $settings_restrictions);
			return $settings_rezip;
		}

		return $settings;
	}

	public function addSettingField($id, $name, $type, $description_tip = null, $default = null)
	{
		$field = array(
			'name' => Plugin::translate($name),
			'type' => $type,
			'id' => $id,
		);

		if (!is_null($description_tip)) {
			$field['desc_tip'] = Plugin::translate($description_tip);
		}

		if (!is_null($default)) {
			$field['default'] = $default;
		}

		return $field;
	}

	public static function getSetting($option, $default = '')
	{
		if (strpos($option, 'rezip-') === false) {
			$option = 'rezip-' . $option;
		}
		$option = get_option($option);

		if (!empty($option)) {
			return $option;
		}

		return $default;
	}

	public static function getFeeName()
	{
		return static::getSetting('fee-name', Plugin::translate('Re-Zip emballage'));
	}

	public static function getTitle()
	{
		// return static::getSetting('title', Plugin::translate('Vil du skåne miljøet?'));
		return static::getSetting('title', Plugin::translate('Nej tak til engangsemballage!'));
	}

	public static function getShortDescription()
	{
		// return static::getSetting('short-description', Plugin::translate('Vælg bæredygtig emballage og spar 100 DKK ved næste handel!'));
		return static::getSetting('short-description', Plugin::translate('Vælg cirkulær emballage og få rabat ved næste køb.'));
	}

	public static function getDescription()
	{
		// return static::getSetting('description', Plugin::translate('Ved fælles indsats kan vi undgå at bruge engangsemballage til din forsendelse. Re-Zip emballage kan returneres i nærmeste postkasse, uden porto, og så kvitterer vi med en kontant rabat på 100 DKK næste gang du handler for over 500 DKK'));
		return static::getSetting('description', Plugin::translate('<p>Sådan fungerer det:</p><ol><li>Tilvælg RE-ZIP´s cirkulære emballage.</li><li>Returner den tomme emballage gratis i en dao pakkeshop ved at scanne QR-koden på emballagen.</li><li>Modtag som tak en rabatkode.</li></ol>'));
	}

	public static function getCheckboxText()
	{
		return static::getSetting('checkbox-text', Plugin::translate('Ja tak!'));
	}

	public static function getFeePrice($including_tax = true)
	{
		$price = static::getSetting('fee-price', 15);

		if (wc_tax_enabled()) {
			if ($including_tax  && !wc_prices_include_tax()) {
				$_taxes = \WC_tax::calc_tax($price, \WC_Tax::get_rates(), FALSE);

				$tax = 0;

				foreach ($_taxes as $_tax) {
					$tax += $_tax;
				}

				return $price + $tax;
			} else if (!$including_tax && wc_prices_include_tax()) {
				$_taxes = \WC_tax::calc_tax($price, \WC_Tax::get_rates(), true);

				$tax = 0;

				foreach ($_taxes as $_tax) {
					$tax += $_tax;
				}

				return $price - $tax;
			}
		}

		return $price;
	}

	public static function getFeePriceExTax()
	{
		return static::getFeePrice(false);
	}

	public static function getPackagingImage()
	{
		$image = static::getSetting('image', Plugin::getImgURL('rezip_badge.png'));

		if (is_numeric($image)) {
			$_image_src = wp_get_attachment_image_src($image, array(150, 150, true));
			$image = $_image_src[0];
		}

		return $image;
	}
}
